/*****************************************************************************
 *	Magna-Power Electronic power supplies
 *	LabWindows/CVI 6.0 Instrument Driver
 *	Original Release: 22. January 2004
 *	By: Magna-Power Electronics, Inc.
 *		PH. 973-263-0017   Fax 973-263-1928
 *
 *	Modification History:
 *
 *
 *	01/04	-	Rev 1.00
 *			-	Instrument Driver Created.
 *
 *	03/04	-	Rev 1.01
 *			-	Added compatability for XXA series.
 *			-	Period and Arm commands are not supported for the XXA series
 *
 *	07/04	-	Rev 1.02
 *			-	Added modulation commands for XXD series.
 *			-	modulation commands are not supported for the XXA series
 *
 *****************************************************************************/
 
#include <visa.h>
#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include "mpexxx.h"

#define mpexxx_REV		"Rev 1.02, 08/2004, CVI 6.0" /*	Instrument driver revision */

/*	inst lmt struct - filled in init */
typedef struct mpe_dat_rng
	{
	ViInt32  model;			/*	model of supply (XXD | XXA) */
	ViReal64 volt_lmt;		/*	voltage lmt of supply */
	ViReal64 curr_lmt;		/*	current lmt of supply */
	ViReal64 ovp_max;		/*	over-voltage max lmt of supply */
	ViReal64 ovp_min;		/*	over-voltage min lmt of supply */
	ViReal64 ocp_max;		/*	over-current max lmt of supply */
	ViReal64 ocp_min;		/*	over-current min lmt of supply */
	ViReal64 per;			/*	period lmt of supply */
	} Inst, *pInst;

/*	struct error message */
typedef struct  mpexxx_stringValPair
	{
	ViStatus stringVal;
	ViString stringName;
	} mpexxx_tStringValPair;

/*	defs for instrument models */
#define MODEL_TYPE_A	0		/*	type a model XXA */
#define MODEL_TYPE_D	1		/*	type a model XXD */

/*	defs for scpi & iee cmds */
#define SET_VOLT	0		/*	idx of voltage setting cmd/query				*/
#define SET_CURR	1		/*	idx of current setting cmd/query				*/
#define SET_OVP		2		/*	idx of over-voltage setting cmd/query			*/
#define SET_OCP		3		/*	idx of over-current setting cmd/query			*/
#define SET_PER		4		/*	idx of period setting cmd/query					*/
#define MEMORY		5		/*	idx of memory setting cmd/query					*/
#define OUTP_START	6		/*	idx of start cmd								*/
#define OUTP_STOP	7		/*	idx of stop cmd									*/
#define OUTP_ARM	8		/*	idx of arm cmd/query							*/
#define OUTP_STAT	9		/*	idx of outpt state query						*/
#define OUTP_CLR	10		/*	idx of alarm clear cmd							*/
#define TRIG_VOLT	11		/*	idx of trigger voltage setting cmd/query		*/
#define TRIG_CURR	12		/*	idx of trigger current setting cmd/query		*/
#define INIT_IMM	13		/*	idx of initiate immediate cmd					*/
#define INIT_CONT	14		/*	idx of initiate continous setting cmd/query		*/
#define TRIGGER		15		/*	idx of trigger immediate cmd					*/
#define ABORT		16		/*	idx of abort cmd								*/
#define MEAS_VOLT	17		/*	idx of measure voltage query					*/
#define MEAS_CURR	18		/*	idx of measure current query					*/
#define SYST_ERR	19		/*	idx of system error query						*/
#define SYST_VER	20		/*	idx of software system version query			*/
#define STAT_OPER	21		/*	idx of status operation register query			*/
#define STAT_QUES	22		/*	idx of status questionable register query		*/
#define CONF_RSEN	23		/*	idx of remote sense cmd/query					*/
#define CONF_ECTL	24		/*	idx of external control cmd/query				*/
#define CONF_ICTL	25		/*	idx of internal control cmd/query				*/
#define CONF_INTE	26		/*	idx of interlock cmd/query						*/
#define CAL_IDN		27		/*	idx of scpi identification string cmd/query		*/
#define CAL_PWD		28		/*	idx of calibration password cmd					*/
#define CAL_POTS	29		/*	idx of calibration potentiometers cmd/query		*/
#define CAL_VSCL	30		/*	idx of voltage scale query						*/
#define CAL_ISCL	31		/*	idx of current scale query						*/
#define CAL_DEFS	32		/*	idx of calibration defaults cmd					*/
#define CAL_STOP	33		/*	idx of ends calibrations setting cmd			*/
#define IEEE_CLS	34		/*	idx of clear interface cmd						*/
#define IEEE_ESE	35		/*	idx of event state enable register cmd/query	*/
#define IEEE_ESR	36		/*	idx of event status reigister query				*/
#define IEEE_IDN	37		/*	idx of ieee identification query				*/
#define IEEE_RCL	38		/*	idx of ieee recall cmd							*/
#define IEEE_RST	39		/*	idx of reset cmd								*/
#define IEEE_SAV	40		/*	idx of ieee save cmd							*/
#define IEEE_SRE	41		/*	idx of service request enable reg cmd/query		*/
#define IEEE_STB	42		/*	idx of status byte register query				*/
#define TABL_TYP	43		/*	idx of modulation settings cmd/query			*/
#define TABL_I_O	44		/*	idx of modulation table cmd/query				*/

/*	strings for cmds	*/
ViString pCmds[] = {
	"VOLT",					/*	actual voltage cmd					*/
	"CURR",					/*	actual current cmd					*/
	"VOLT:PROT",			/*	actual over-voltage cmd				*/
	"CURR:PROT",			/*	actual over-current cmd				*/
	"PER",					/*	actual period cmd					*/
	"MEM",					/*	actual memory cmd					*/
	"OUTP:START",			/*	actual start cmd					*/
	"OUTP:STOP",			/*	actual stop cmd						*/
	"OUTP:ARM",				/*	actual arm cmd						*/
	"OUTP",					/*	actual output state cmd				*/
	"OUTP:PROT:CLE",		/*	actual alarm clear cmd				*/
	"VOLT:TRIG",			/*	actual trigger voltage cmd			*/
	"CURR:TRIG",			/*	actual trigger current cmd			*/
	"INIT:IMM",				/*	actual initiate immediate cmd		*/
	"INIT:CONT",			/*	actual initiate continous cmd		*/
	"TRIG:IMM",				/*	actual trigger immediate cmd		*/
	"ABOR",					/*	actual abort cmd					*/
	"MEAS:VOLT",			/*	actual measure voltage cmd			*/
	"MEAS:CURR",			/*	actual measure current cmd			*/
	"SYST:ERR",				/*	actual system error cmd				*/
	"SYST:VERS",			/*	actual system version cmd			*/
	"STAT:OPER:COND",		/*	actual status operation cmd			*/
	"STAT:QUES:COND",		/*	actual status questionable cmd		*/
	"CONF:REM:SENS",		/*	actual remote sense cmd				*/
	"CONF:CONT:EXT",		/*	actual external control cmd			*/
	"CONF:CONT:INT",		/*	actual internal control cmd			*/
	"CONF:INTE",			/*	actual interlock cmd				*/
	"CAL:IDN",				/*	actual scpi identification cmd		*/
	"CAL:PASS",				/*	actual calibration password cmd		*/
	"CAL:POT",				/*	actual calibration pot cmd			*/
	"CAL:SCAL:VOLT",		/*	actual voltage scale cmd			*/
	"CAL:SCAL:CURR",		/*	actual current scale cmd			*/
	"CAL:DEF",				/*	actual default potentiometer cmd	*/
	"CAL:STOP",				/*	actual calibration stop cmd			*/
	"*CLS",					/*	actual ieee clear interface cmd		*/
	"*ESE",					/*	actual event status enable cmd		*/
	"*ESR",					/*	actual event status register cmd	*/
	"*IDN",					/*	actual ieee identification cmd		*/
	"*RCL",					/*	actual recall cmd					*/
	"*RST",					/*	actual reset cmd					*/
	"*SAV",					/*	actual save cmd						*/
	"*SRE",					/*	actual service request enable cmd	*/
	"*STB",					/*	actual status byte register cmd		*/
	"MOD:TYPE:SEL",			/*	actual modulation settings			*/
	"MOD:TABL"				/*	actual modulation table cmd			*/
	};


/*----------------------------------------------------------------------------------
					non-exportable routines for mpe instrument
  ----------------------------------------------------------------------------------*/

ViStatus _VI_FUNC mpexxx_initCleanUp(ViSession instRMSess, ViPSession vi, ViStatus curStatus);
ViStatus _VI_FUNC mpexxx_set_rst (ViSession vi);

/*----------------------------------------------------------------------------------
						value chk routines for mpe instrument
  ----------------------------------------------------------------------------------*/
ViBoolean _VI_FUNC mpexxx_Chk_ViBoolean(ViBoolean val);
ViBoolean _VI_FUNC mpexxx_Chk_ViInt16(ViInt16 val, ViInt16 max, ViInt16 min);
ViBoolean _VI_FUNC mpexxx_Chk_ViInt32(ViInt32 val, ViInt32 max, ViInt32 min);
ViBoolean _VI_FUNC mpexxx_Chk_ViReal64(ViReal64 val, ViReal64 max, ViReal64 min);

/*----------------------------------------------------------------------------------
						output routines for mpe instrument
  ----------------------------------------------------------------------------------*/
ViStatus _VI_FUNC mpexxx_Cmd_ViReal64(ViSession vi, ViInt16 cmd, ViReal64 val);
ViStatus _VI_FUNC mpexxx_Cmd_ViInt16(ViSession vi, ViInt16 cmd, ViInt16 val);
ViStatus _VI_FUNC mpexxx_Cmd_TwoInts(ViSession vi, ViInt16 cmd, ViInt16 val1, ViInt16 val2);
ViStatus _VI_FUNC mpexxx_Cmd_ViBoolean(ViSession vi, ViInt16 cmd, ViBoolean val);
ViStatus _VI_FUNC mpexxx_Cmd_ViString(ViSession vi, ViInt16 cmd, ViString pBuf);
ViStatus _VI_FUNC mpexxx_Cmd(ViSession vi, ViInt16 cmd);

/*----------------------------------------------------------------------------------
						query routines for mpe instrument
  ----------------------------------------------------------------------------------*/
ViStatus _VI_FUNC mpexxx_Get_MAX(ViSession vi, ViInt16 cmd, ViPReal64 val);
ViStatus _VI_FUNC mpexxx_Get_MIN(ViSession vi, ViInt16 cmd, ViPReal64 val);
ViStatus _VI_FUNC mpexxx_Get_ViReal64(ViSession vi, ViInt16 cmd, ViPReal64 val);
ViStatus _VI_FUNC mpexxx_Get_ViInt16(ViSession vi, ViInt16 cmd, ViPInt16 val);
ViStatus _VI_FUNC mpexxx_Get_PotVal(ViSession vi, ViInt16 cmd, ViInt16 pot, ViPInt16 val);
ViStatus _VI_FUNC mpexxx_Get_Bool(ViSession vi, ViInt16 cmd, ViPBoolean val);
ViStatus _VI_FUNC mpexxx_Get_String(ViSession vi, ViInt16 cmd, ViString pBuf, ViInt16 buf_siz);

/*----------------------------------------------------------------------------------
  ------------------------------- exportable routines ------------------------------
  ----------------------------------------------------------------------------------*/

/*----------------------------------------------------------------------------------
						system routines for mpe instrument
  ----------------------------------------------------------------------------------*/
/*	mpexxx_init
		inits visa, lmt struct & resets if desirable
		resourceString	-	descriptor for comm bus
		ID_Query		-	id query discrete
		Reset			-	reset discrete
		vi				-	ptr to session handle */
ViStatus _VI_FUNC mpexxx_init(ViRsrc resourceString, ViBoolean ID_Query,
													ViBoolean Reset, ViSession *vi)
	{
	ViStatus stat;
	ViSession rmSess = 0;
	ViReal64 val;
	ViInt32 baud;
	ViChar buf[110] = {0};
	ViInt16	cnt;
	pInst inst = NULL;

	/*	opens default inst sess	*/
	if ((stat = viOpenDefaultRM(&rmSess)) < VI_SUCCESS)
		return stat;

	/*	opens specific inst session	*/
	if ((stat = viOpen(rmSess, resourceString, VI_NULL, VI_NULL, vi)) < VI_SUCCESS)
		{
		viClose(rmSess);
		return stat;
		}

	/*	sets timo value	*/
	if ((stat = viSetAttribute(*vi, VI_ATTR_TMO_VALUE, 2000)) < VI_SUCCESS)
		return mpexxx_initCleanUp(rmSess, vi, stat);

	/*	sets baud rate - only used for serial port	*/
	if ((stat = viGetAttribute(*vi, VI_ATTR_ASRL_BAUD, &baud)) == VI_SUCCESS)
		{
		if ((stat = viSetAttribute(*vi, VI_ATTR_ASRL_BAUD, 19200)) < VI_SUCCESS)
			return mpexxx_initCleanUp(rmSess, vi, stat);
		}

	/*	sets term char	*/
	if ((stat = viSetAttribute(*vi, VI_ATTR_TERMCHAR, 0x0A)) < VI_SUCCESS)
		return mpexxx_initCleanUp(rmSess, vi, stat);

	/*	sets term char enabled - needed to determine end of transaction	*/
	if ((stat = viSetAttribute(*vi, VI_ATTR_TERMCHAR_EN, VI_TRUE)) < VI_SUCCESS)
		return mpexxx_initCleanUp(rmSess, vi, stat);

	/*	allocates mem for lmt struct	*/
	if ((inst = (pInst)malloc(sizeof(Inst))) == NULL )
		return mpexxx_initCleanUp(rmSess, vi, stat);

	/*	sets ptr adr in vi session	*/
    if ((stat = viSetAttribute (*vi, VI_ATTR_USER_DATA, (ViUInt32)inst)) != 0)
		return mpexxx_initCleanUp(rmSess, vi, stat);

	/*	finds max volt for inst	*/
	if ((stat = mpexxx_Get_MAX(*vi, SET_VOLT, &val)) < VI_SUCCESS)
		return mpexxx_initCleanUp(rmSess, vi, stat);
	inst->volt_lmt = val;

	/*	finds max curr for inst	*/
	if ((stat = mpexxx_Get_MAX(*vi, SET_CURR, &val)) < VI_SUCCESS)
		return mpexxx_initCleanUp(rmSess, vi, stat);
	inst->curr_lmt = val;

	/*	sets lmt struct from recv vals	*/
	inst->ovp_max = inst->volt_lmt * 1.1;
	inst->ovp_min = inst->volt_lmt * 0.1;
	inst->ocp_max = inst->curr_lmt * 1.1;
	inst->ocp_min = inst->curr_lmt * 0.1;
	inst->per = 9999.0;
	/*	defaults to type a - chg if desired for tpye d */
	inst->model = MODEL_TYPE_A;

    /*	Identification Query	*/
    if (ID_Query)
		{
		/*	issues cmd & returns buf	*/
		if ((stat = mpexxx_Get_String(*vi, IEEE_IDN, buf, 110)) < 0)
			return mpexxx_initCleanUp(rmSess, vi, stat);

		/*	forces lower case	*/
		for (cnt = 0; buf[cnt] && cnt < sizeof(buf); cnt++)
			buf[cnt] = tolower(buf[cnt]);
		
		/*	search for name	*/
		/*	if you changed the *idn string you'll need to modify this line	*/
		if (strstr(buf, "magna-power electronics") == NULL) 
			return mpexxx_initCleanUp(rmSess, vi, stat);
		
		/*	search for type	*/
		/*	if you changed the *idn string you'll need to modify this block	*/
		if (strstr(buf, "pqd") != NULL)
			inst->model = MODEL_TYPE_D;
		else if (strstr(buf, "sqd") != NULL)
			inst->model = MODEL_TYPE_D;
		else if (strstr(buf, "mqd") != NULL)
			inst->model = MODEL_TYPE_D;
		else if (strstr(buf, "mtd") != NULL)
			inst->model = MODEL_TYPE_D;
		else
			inst->model = MODEL_TYPE_A;
	    }

	/*	resets inst	*/
	if (Reset)
		{
		if ((stat = mpexxx_reset(*vi)) < VI_SUCCESS)
			return mpexxx_initCleanUp(rmSess, vi, stat);
		} 

	return stat;
	}

/*	mpexxx_close
		closes session and frees memory
		vi	-	specific session	*/
ViStatus _VI_FUNC mpexxx_close (ViSession vi)
	{
	ViStatus stat;
	ViSession rmSess;
	pInst inst = NULL;

	/*	retrieves structure ptr for lmts	*/
    if ((stat = viGetAttribute (vi, VI_ATTR_USER_DATA, &inst)) < 0)
        return stat;

	/*	frees memory	*/
	if (inst != NULL) 
		free(inst);

	/*	defaults var	*/
	inst = NULL;

	/*	gets resource manager	*/
	if ((stat = viGetAttribute(vi, VI_ATTR_RM_SESSION, &rmSess)) < VI_SUCCESS)
		return stat;

	/*	closes sessions	*/
	stat = viClose(vi);
	viClose(rmSess);

	return stat;
}

/*----------------------------------------------------------------------------------
						exported cmds for mpe instrument
  ----------------------------------------------------------------------------------*/

/*	mpexxx_conf
		performs complete set point settings
		vi				-	session handle
		remoteSense		-	remote sense discrete
		internalControl	-	internal control discrete
		externalControl	-	external control discrete
		interlock		-	interlock discrete	*/
ViStatus _VI_FUNC mpexxx_conf (ViSession vi, ViBoolean remoteSense,
                               ViBoolean internalControl,
                               ViBoolean externalControl, ViBoolean interlock)
	{
	ViStatus stat;

	/*	chks ranges of the parameters - returns on first failure	*/
	if (mpexxx_Chk_ViBoolean(remoteSense))
		return VI_ERROR_PARAMETER2;

	if (mpexxx_Chk_ViBoolean(internalControl))
		return VI_ERROR_PARAMETER3;

	if (mpexxx_Chk_ViBoolean(externalControl))
		return VI_ERROR_PARAMETER4;

	if (mpexxx_Chk_ViBoolean(interlock))
		return VI_ERROR_PARAMETER5;

	/*	writes configure settings to inst - returns on first failure	*/
	if ((stat = mpexxx_Cmd_ViBoolean(vi, CONF_RSEN, remoteSense)) < VI_SUCCESS)
		return stat;

	if ((stat = mpexxx_Cmd_ViBoolean(vi, CONF_ICTL, internalControl)) < VI_SUCCESS)
		return stat;

	if ((stat = mpexxx_Cmd_ViBoolean(vi, CONF_ECTL, externalControl)) < VI_SUCCESS)
		return stat;

	stat = mpexxx_Cmd_ViBoolean(vi, CONF_INTE, interlock);

	return stat;
	}

/*	mpexxx_set_rem_sen
		sets the state of remote sense
		vi			-	session handle
		remoteSense	-	remote sense discrete	*/
ViStatus _VI_FUNC mpexxx_set_rem_sen (ViSession vi, ViBoolean remoteSense)
	{
	ViStatus stat;

	/*	chks for valid bool	*/
	if (mpexxx_Chk_ViBoolean(remoteSense))
		return VI_ERROR_PARAMETER2;

	/*	issues remots sense cmd	*/
	stat = mpexxx_Cmd_ViBoolean(vi, CONF_RSEN, remoteSense);

	return stat;
	}

/*	mpexxx_get_rem_sen
		gets the state of external control
		vi			-	session handle
		remoteSense	-	ptr remote sense discrete	*/
ViStatus _VI_FUNC mpexxx_get_rem_sen (ViSession vi, ViBoolean *remoteSense)
	{
	ViStatus stat;

	/*	issues cmd and acquires response	*/
	stat = mpexxx_Get_Bool(vi, CONF_RSEN, remoteSense);

	return stat;
	}

/*	mpexxx_set_int_ctrl
		sets the state of the internal control
		vi				-	session handle
		internalControl	-	internal control discrete	*/
ViStatus _VI_FUNC mpexxx_set_int_ctrl (ViSession vi, ViBoolean internalControl)
	{
	ViStatus stat;

	/*	chks for valid bool	*/
	if (mpexxx_Chk_ViBoolean(internalControl))
		return VI_ERROR_PARAMETER2;

	/*	issues internal control cmd	*/
	stat = mpexxx_Cmd_ViBoolean(vi, CONF_ICTL, internalControl);

	return stat;
	}

/*	mpexxx_get_int_ctrl
		gets the state of internal control
		vi				-	session handle
		internalControl	-	ptr internal control discrete	*/
ViStatus _VI_FUNC mpexxx_get_int_ctrl (ViSession vi, ViBoolean *internalControl)
	{
	ViStatus stat;

	/*	issues cmd and acquires response	*/
	stat = mpexxx_Get_Bool(vi, CONF_ICTL, internalControl);

	return stat;
	}

/*	mpexxx_set_ext_ctrl
		sets the state of the external control
		vi				-	session handle
		externalControl	-	external control state	*/
ViStatus _VI_FUNC mpexxx_set_ext_ctrl (ViSession vi, ViBoolean externalControl)
	{
	ViStatus stat;

	/*	chks for valid bool	*/
	if (mpexxx_Chk_ViBoolean(externalControl))
		return VI_ERROR_PARAMETER2;

	/*	issues external control cmd	*/
	stat = mpexxx_Cmd_ViBoolean(vi, CONF_ECTL, externalControl);

	return stat;
	}

/*	mpexxx_get_ext_ctrl
		gets the state of external control
		vi				-	session handle
		externalControl	-	ptr discrete for external control state	*/
ViStatus _VI_FUNC mpexxx_get_ext_ctrl (ViSession vi, ViBoolean *externalControl)
	{
	ViStatus stat;

	/*	issues cmd and acquires response	*/
	stat = mpexxx_Get_Bool(vi, CONF_ECTL, externalControl);

	return stat;
	}

/*	mpexxx_set_interlock
		sets the state of the interlock
		vi			-	session handle
		interlock	-	interlock discrete	*/
ViStatus _VI_FUNC mpexxx_set_interlock (ViSession vi, ViBoolean interlock)
	{
	ViStatus stat;

	/*	chks for valid bool	*/
	if (mpexxx_Chk_ViBoolean(interlock))
		return VI_ERROR_PARAMETER2;

	/*	issues the interlock cmd	*/
	stat = mpexxx_Cmd_ViBoolean(vi, CONF_INTE, interlock);

	return stat;
	}

/*	mpexxx_get_interlock
		gets the interlock state
		vi			-	session handle
		interlock	-	ptr to state of interlock	*/
ViStatus _VI_FUNC mpexxx_get_interlock (ViSession vi, ViBoolean *interlock)
	{
	ViStatus stat;

	/*	issues cmd and acquires response	*/
	stat = mpexxx_Get_Bool(vi, CONF_INTE, interlock);

	return stat;
	}

/*	mpexxx_set_pnts_xxd
		sets the set points for the xxd series
		vi		-	session handle
		voltage	-	voltage setting
		current	-	current setting
		OVP		-	over-voltage trip setting
		OCP		-	over-current trip setting
		period	-	period setting	*/
ViStatus _VI_FUNC mpexxx_set_pnts_xxd (ViSession vi, ViReal64 voltage,
                                   ViReal64 current, ViReal64 OVP, ViReal64 OCP,
                                   ViReal64 period)
	{
	ViStatus stat;
	pInst inst = NULL;
        
	/*	retrieves structure ptr for lmts	*/
    if ((stat = viGetAttribute (vi, VI_ATTR_USER_DATA, &inst)) < 0)
        return stat;

	/*	chks for valid lmt struct	*/
	if (inst == NULL) 
		return MPEXXX_ERR_DRIVER_DATA;

	/*	chks ranges of the parameters - returns on first failure	*/
	if (mpexxx_Chk_ViReal64(voltage, inst->volt_lmt, 0.0))
		return VI_ERROR_PARAMETER2;

	if (mpexxx_Chk_ViReal64(current, inst->curr_lmt, 0.0))
		return VI_ERROR_PARAMETER3;

	if (mpexxx_Chk_ViReal64(OVP, inst->ovp_max, inst->ovp_min))
		return VI_ERROR_PARAMETER4;

	if (mpexxx_Chk_ViReal64(OCP, inst->ocp_max, inst->ocp_min))
		return VI_ERROR_PARAMETER5;

	if (mpexxx_Chk_ViReal64(period, inst->per, 0.0))
		return VI_ERROR_PARAMETER6;

	/*	writes set points to inst - returns on first failure	*/
	if ((stat = mpexxx_Cmd_ViReal64(vi, SET_VOLT, voltage)) < VI_SUCCESS)
		return stat;

	if ((stat = mpexxx_Cmd_ViReal64(vi, SET_CURR, current)) < VI_SUCCESS)
		return stat;

	if ((stat = mpexxx_Cmd_ViReal64(vi, SET_OVP, OVP)) < VI_SUCCESS)
		return stat;

	if ((stat = mpexxx_Cmd_ViReal64(vi, SET_OCP, OCP)) < VI_SUCCESS)
		return stat;

	/*	chks for correct inst type	*/
	if (inst->model == MODEL_TYPE_A) 
		return stat;

	stat = mpexxx_Cmd_ViReal64(vi, SET_PER, period);

	return stat;
	}

/*	mpexxx_set_pnts_xxa
		sets the set points for the xxa series
		vi		-	session handle
		voltage	-	voltage setting
		current	-	current setting
		OVP		-	over-voltage trip setting
		OCP		-	over-current trip setting	*/
ViStatus _VI_FUNC mpexxx_set_pnts_xxa (ViSession vi, ViReal64 voltage,
                                   ViReal64 current, ViReal64 OVP, ViReal64 OCP)
	{
	ViStatus stat;
	pInst inst = NULL;
        
	/*	retrieves structure ptr for lmts	*/
    if ((stat = viGetAttribute (vi, VI_ATTR_USER_DATA, &inst)) < 0)
        return stat;

	/*	chks for valid lmt struct	*/
	if (inst == NULL) 
		return MPEXXX_ERR_DRIVER_DATA;

	/*	chks ranges of the parameters - returns on first failure	*/
	if (mpexxx_Chk_ViReal64(voltage, inst->volt_lmt, 0.0))
		return VI_ERROR_PARAMETER2;

	if (mpexxx_Chk_ViReal64(current, inst->curr_lmt, 0.0))
		return VI_ERROR_PARAMETER3;

	if (mpexxx_Chk_ViReal64(OVP, inst->ovp_max, inst->ovp_min))
		return VI_ERROR_PARAMETER4;

	if (mpexxx_Chk_ViReal64(OCP, inst->ocp_max, inst->ocp_min))
		return VI_ERROR_PARAMETER5;

	/*	writes set points to inst - returns on first failure	*/
	if ((stat = mpexxx_Cmd_ViReal64(vi, SET_VOLT, voltage)) < VI_SUCCESS)
		return stat;

	if ((stat = mpexxx_Cmd_ViReal64(vi, SET_CURR, current)) < VI_SUCCESS)
		return stat;

	if ((stat = mpexxx_Cmd_ViReal64(vi, SET_OVP, OVP)) < VI_SUCCESS)
		return stat;

	stat = mpexxx_Cmd_ViReal64(vi, SET_OCP, OCP);

	return stat;
	}

/*	mpexxx_set_volt_curr
		sets voltage and current of supply
		vi		-	session handle
		voltage	-	voltage setting
		current	-	current setting	*/
ViStatus _VI_FUNC mpexxx_set_volt_curr (ViSession vi, ViReal64 voltage,
                                        ViReal64 current)
	{
	ViStatus stat;
	pInst inst = NULL;
        
	/*	retrieves structure ptr for lmts	*/
    if ((stat = viGetAttribute (vi, VI_ATTR_USER_DATA, &inst)) < 0)
        return stat;

	/*	chks for valid lmt struct	*/
	if (inst == NULL) 
		return MPEXXX_ERR_DRIVER_DATA;

	/*	chks ranges of the parameters - returns on first failure	*/
	if (mpexxx_Chk_ViReal64(voltage, inst->volt_lmt, 0.0))
		return VI_ERROR_PARAMETER2;

	if (mpexxx_Chk_ViReal64(current, inst->curr_lmt, 0.0))
		return VI_ERROR_PARAMETER3;

	/*	writes volt & curr to inst - returns on first failure	*/
	if ((stat = mpexxx_Cmd_ViReal64(vi, SET_VOLT, voltage)) < VI_SUCCESS)
		return stat;

	stat = mpexxx_Cmd_ViReal64(vi, SET_CURR, current);

	return stat;
	}

/*	mpexxx_set_ovp_ocp
		sets over-voltage and over-current trip of supply
		vi	-	session handle
		OVP	-	over-voltage trip setting
		OCP	-	over-current trip setting	*/
ViStatus _VI_FUNC mpexxx_set_ovp_ocp (ViSession vi, ViReal64 OVP, ViReal64 OCP)
	{
	ViStatus stat;
	pInst inst = NULL;
        
	/*	retrieves structure ptr for lmts	*/
    if ((stat = viGetAttribute (vi, VI_ATTR_USER_DATA, &inst)) < 0)
        return stat;

	/*	chks for valid lmt struct	*/
	if (inst == NULL) 
		return MPEXXX_ERR_DRIVER_DATA;

	/*	chks ranges of the parameters - returns on first failure	*/
	if (mpexxx_Chk_ViReal64(OVP, inst->ovp_max, inst->ovp_min))
		return VI_ERROR_PARAMETER2;

	if (mpexxx_Chk_ViReal64(OCP, inst->ocp_max, inst->ocp_min))
		return VI_ERROR_PARAMETER3;

	/*	writes protection to inst - returns on first failure	*/
	if ((stat = mpexxx_Cmd_ViReal64(vi, SET_OVP, OVP)) < VI_SUCCESS)
		return stat;

	stat = mpexxx_Cmd_ViReal64(vi, SET_OCP, OCP);

	return stat;
	}

/*	mpexxx_set_volt
		sets voltage of supply
		vi		-	session handle
		voltage	-	voltage setting	*/
ViStatus _VI_FUNC mpexxx_set_volt (ViSession vi, ViReal64 voltage)
	{
	ViStatus stat;
	pInst inst = NULL;
        
	/*	retrieves structure ptr for lmts	*/
    if ((stat = viGetAttribute (vi, VI_ATTR_USER_DATA, &inst)) < 0)
        return stat;

	/*	chks for valid lmt struct	*/
	if (inst == NULL) 
		return MPEXXX_ERR_DRIVER_DATA;

	/*	chks ranges of the parameter	*/
	if (mpexxx_Chk_ViReal64(voltage, inst->volt_lmt, 0.0))
		return VI_ERROR_PARAMETER2;

	/*	issues cmd and acquires response	*/
	stat = mpexxx_Cmd_ViReal64(vi, SET_VOLT, voltage);

	return stat;
	}

/*	mpexxx_get_volt
		gets voltage of supply
		vi		-	session handle
		voltage	-	ptr to voltage setting	*/
ViStatus _VI_FUNC mpexxx_get_volt (ViSession vi, ViReal64 *voltage)
	{
	ViStatus stat;

	/*	issues cmd and acquires response	*/
	stat = mpexxx_Get_ViReal64(vi, SET_VOLT, voltage);

	return stat;
	}

/*	mpexxx_set_curr
		sets the set point for current
		vi		-	session handle
		current	-	current set point value	*/
ViStatus _VI_FUNC mpexxx_set_curr (ViSession vi, ViReal64 current)
	{
	ViStatus stat;
	pInst inst = NULL;
        
	/*	retrieves structure ptr for lmts	*/
    if ((stat = viGetAttribute (vi, VI_ATTR_USER_DATA, &inst)) < 0)
        return stat;

	/*	chks for valid lmt struct	*/
	if (inst == NULL) 
		return MPEXXX_ERR_DRIVER_DATA;

	/*	chks ranges of the parameter	*/
	if (mpexxx_Chk_ViReal64(current, inst->curr_lmt, 0.0))
		return VI_ERROR_PARAMETER2;

	/*	issues cmd and acquires response	*/
	stat = mpexxx_Cmd_ViReal64(vi, SET_CURR, current);

	return stat;
	}

/*	mpexxx_get_curr
		returns the set point for current
		vi		-	session handle
		amps	-	ptr used to return current	*/
ViStatus _VI_FUNC mpexxx_get_curr (ViSession vi, ViReal64 *current)
	{
	ViStatus stat;

	/*	issues cmd and acquires response	*/
	stat = mpexxx_Get_ViReal64(vi, SET_CURR, current);

	return stat;
	}

/*	mpexxx_set_ovp
		returns the set point for over-voltage
		vi	-	session handle
		OVP	-	value to set over-voltage trip	*/
ViStatus _VI_FUNC mpexxx_set_ovp (ViSession vi, ViReal64 OVP)
	{
	ViStatus stat;
	pInst inst = NULL;
        
	/*	retrieves structure ptr for lmts	*/
    if ((stat = viGetAttribute (vi, VI_ATTR_USER_DATA, &inst)) < 0)
        return stat;

	/*	chks for valid lmt struct	*/
	if (inst == NULL) 
		return MPEXXX_ERR_DRIVER_DATA;

	/*	chks ranges of the parameter	*/
	if (mpexxx_Chk_ViReal64(OVP, inst->ovp_max, inst->ovp_min))
		return VI_ERROR_PARAMETER2;

	/*	issues cmd and acquires response	*/
	stat = mpexxx_Cmd_ViReal64(vi, SET_OVP, OVP);

	return stat;
	}

/*	mpexxx_Get_Ovp
		returns the set point for over-voltage
		vi	-	session handle
		OVP	-	ptr used to return over-voltage	*/
ViStatus _VI_FUNC mpexxx_get_ovp (ViSession vi, ViReal64 *OVP)
	{
	ViStatus stat;

	/*	issues cmd and acquires response	*/
	stat = mpexxx_Get_ViReal64(vi, SET_OVP, OVP);

	return stat;
	}

/*	mpexxx_set_ocp
		sets the set point for over-current
		vi	-	session handle
		OVP	-	value to set over-current trip	*/
ViStatus _VI_FUNC mpexxx_set_ocp (ViSession vi, ViReal64 OCP)
	{
	ViStatus stat;
	pInst inst = NULL;
        
	/*	retrieves structure ptr for lmts	*/
    if ((stat = viGetAttribute (vi, VI_ATTR_USER_DATA, &inst)) < 0)
        return stat;

	/*	chks for valid lmt struct	*/
	if (inst == NULL) 
		return MPEXXX_ERR_DRIVER_DATA;

	/*	chks ranges of the parameter	*/
	if (mpexxx_Chk_ViReal64(OCP, inst->ocp_max, inst->ocp_min))
		return VI_ERROR_PARAMETER2;

	/*	issues cmd and acquires response	*/
	stat = mpexxx_Cmd_ViReal64(vi, SET_OCP, OCP);

	return stat;
	}

/*	mpexxx_Get_Ocp
		returns the set point for over-current
		vi	-	session handle
		ocp	-	ptr used to return over-current	*/
ViStatus _VI_FUNC mpexxx_get_ocp (ViSession vi, ViReal64 *OCP)
	{
	ViStatus stat;

	/*	issues cmd and acquires response	*/
	stat = mpexxx_Get_ViReal64(vi, SET_OCP, OCP);

	return stat;
	}

/*	mpexxx_set_period
		sets the set point for period
		vi		-	session handle
		period	-	value of period	*/
ViStatus _VI_FUNC mpexxx_set_period (ViSession vi, ViReal64 period)
	{
	ViStatus stat;
	pInst inst = NULL;
        
	/*	retrieves structure ptr for lmts	*/
    if ((stat = viGetAttribute (vi, VI_ATTR_USER_DATA, &inst)) < 0)
        return stat;

	/*	chks for valid lmt struct	*/
	if (inst == NULL) 
		return MPEXXX_ERR_DRIVER_DATA;

	/*	chks for correct inst type	*/
	if (inst->model == MODEL_TYPE_A) 
		return MPEXXX_WARNING_MODEL_DATA;

	/*	chks range of the parameter - returns on failure	*/
	if (mpexxx_Chk_ViReal64(period, inst->per, 0.0))
		return VI_ERROR_PARAMETER2;

	/*	writes period to supply	*/
	stat = mpexxx_Cmd_ViReal64(vi, SET_PER, period);

	return stat;
	}

/*	mpexxx_get_period
		gets current period of supply
		vi		-	session handle
		period	-	ptr to period	*/
ViStatus _VI_FUNC mpexxx_get_period (ViSession vi, ViReal64 *period)
	{
	ViStatus stat;
	pInst inst = NULL;
        
	/*	retrieves structure ptr for lmts	*/
    if ((stat = viGetAttribute (vi, VI_ATTR_USER_DATA, &inst)) < 0)
        return stat;

	/*	chks for valid lmt struct	*/
	if (inst == NULL) 
		return MPEXXX_ERR_DRIVER_DATA;

	/*	chks for correct inst type	*/
	if (inst->model == MODEL_TYPE_A) 
		return MPEXXX_WARNING_MODEL_DATA;

	/*	issues cmd and acquires response	*/
	stat = mpexxx_Get_ViReal64(vi, SET_PER, period);

	return stat;
	}

/*	mpexxx_set_mem
		sets the current memory location
		vi	-	session handle
		mem	-	value for memory location	*/
ViStatus _VI_FUNC mpexxx_set_mem (ViSession vi, ViInt16 mem)
	{
	ViStatus stat;
	pInst inst = NULL;
        
	/*	retrieves structure ptr for lmts	*/
    if ((stat = viGetAttribute (vi, VI_ATTR_USER_DATA, &inst)) < 0)
        return stat;

	/*	chks for valid lmt struct	*/
	if (inst == NULL) 
		return MPEXXX_ERR_DRIVER_DATA;

	/*	chks range of the parameter - returns on failure	*/
	if (mpexxx_Chk_ViInt16(mem, 99, 0))
		return VI_ERROR_PARAMETER2;

	/*	writes mem location	*/
	stat = mpexxx_Cmd_ViInt16(vi, MEMORY, mem);

	return stat;
	}

/*	mpexxx_get_mem
		gets current memory location
		vi	-	session handle
		mem	-	ptr to value for memory location	*/
ViStatus _VI_FUNC mpexxx_get_mem (ViSession vi, ViInt16 *mem)
	{
	ViStatus stat;

	/*	issues cmd and acquires response	*/
	stat = mpexxx_Get_ViInt16(vi, MEMORY, mem);

	return stat;
	}

/*	mpexxx_set_arm
		enables/disables arm of supply
		vi	-	session handle
		arm	-	discrete for arm setting	*/
ViStatus _VI_FUNC mpexxx_set_arm (ViSession vi, ViBoolean arm)
	{
	ViStatus stat;
	pInst inst = NULL;
        
	/*	retrieves structure ptr for lmts	*/
    if ((stat = viGetAttribute (vi, VI_ATTR_USER_DATA, &inst)) < 0)
        return stat;

	/*	chks for valid lmt struct	*/
	if (inst == NULL) 
		return MPEXXX_ERR_DRIVER_DATA;

	/*	chks for correct inst type	*/
	if (inst->model == MODEL_TYPE_A) 
		return MPEXXX_WARNING_MODEL_DATA;

	/*	chks rng of input	*/
	if (mpexxx_Chk_ViBoolean(arm))
		return VI_ERROR_PARAMETER2;

	/*	output arm state	*/
	stat = mpexxx_Cmd_ViBoolean(vi, OUTP_ARM, arm);

	return stat;
	}

/*	mpexxx_get_arm
		gets the state of the arm setting
		vi	-	session handle
		arm	-	ptr to discrete for arm setting	*/
ViStatus _VI_FUNC mpexxx_get_arm (ViSession vi, ViBoolean *arm)
	{
	ViStatus stat;
	pInst inst = NULL;
        
	/*	retrieves structure ptr for lmts	*/
    if ((stat = viGetAttribute (vi, VI_ATTR_USER_DATA, &inst)) < 0)
        return stat;

	/*	chks for valid lmt struct	*/
	if (inst == NULL) 
		return MPEXXX_ERR_DRIVER_DATA;

	/*	chks for correct inst type	*/
	if (inst->model == MODEL_TYPE_A) 
		return MPEXXX_WARNING_MODEL_DATA;

	/*	issues cmd and acquires response	*/
	stat = mpexxx_Get_Bool(vi, OUTP_ARM, arm);

	return stat;
	}

/*	mpexxx_set_outp
		enables/disables the output of the supply
		vi			-	session handle
		powerState	-	discrete for output of the power supply	*/
ViStatus _VI_FUNC mpexxx_set_outp (ViSession vi, ViBoolean powerState)
	{
	ViStatus stat;

	/*	chks rng of input	*/
	if (mpexxx_Chk_ViBoolean(powerState))
		return VI_ERROR_PARAMETER2;

	/*	energizes supply	*/
	if (powerState == VI_TRUE)
		stat = mpexxx_Cmd(vi, OUTP_START);
	else
		stat = mpexxx_Cmd(vi, OUTP_STOP);

	return stat;
	}

/*	mpexxx_clr_outp
		clears the alarm state
		vi	-	session handle	*/
ViStatus _VI_FUNC mpexxx_clr_outp (ViSession vi)
	{
	ViStatus stat;

	/*	clears alarm latches	*/
	stat = mpexxx_Cmd(vi, OUTP_CLR);

	return stat;
	}

/*	mpexxx_get_outp
		gets the output state of supply
		vi			-	session handlehandle
		powerState	-	ptr to the output state	*/
ViStatus _VI_FUNC mpexxx_get_outp (ViSession vi, ViBoolean *powerState)
	{
	ViStatus stat;

	/*	issues cmd and acquires response	*/
	stat = mpexxx_Get_Bool(vi, OUTP_STAT, powerState);

	return stat;
	}

/*	mpexxx_set_trigger
		sets the trigger state of supply
		vi					-	session handle
		triggerVoltage		-	value for trigger voltage setting
		triggerCurrent		-	value for trigger current setting
		initiateContinuous	-	value for initiate continous	*/
ViStatus _VI_FUNC mpexxx_set_trigger (ViSession vi, ViReal64 triggerVoltage,
                                      ViReal64 triggerCurrent,
                                      ViBoolean initiateContinuous)
	{
	ViStatus stat;
	pInst inst = NULL;
        
	/*	retrieves structure ptr for lmts	*/
    if ((stat = viGetAttribute (vi, VI_ATTR_USER_DATA, &inst)) < 0)
        return stat;

	/*	chks for valid lmt struct	*/
	if (inst == NULL) 
		return MPEXXX_ERR_DRIVER_DATA;

	/*	chks range of the parameter - returns on failure	*/
	if (mpexxx_Chk_ViReal64(triggerVoltage, inst->volt_lmt, 0.0))
		return VI_ERROR_PARAMETER2;

	if (mpexxx_Chk_ViReal64(triggerCurrent, inst->curr_lmt, 0.0))
		return VI_ERROR_PARAMETER3;

	if (mpexxx_Chk_ViBoolean(initiateContinuous))
		return VI_ERROR_PARAMETER4;

	/*	writes trigger settings to inst - returns on first failure	*/
	if ((stat = mpexxx_Cmd_ViReal64(vi, TRIG_VOLT, triggerVoltage)) < VI_SUCCESS)
		return stat;

	if ((stat = mpexxx_Cmd_ViReal64(vi, TRIG_CURR, triggerCurrent)) < VI_SUCCESS)
		return stat;

	stat = mpexxx_Cmd_ViBoolean(vi, INIT_CONT, initiateContinuous);

	return stat;
	}

/*	mpexxx_abort
		sends an abort to stop current triggering
		vi	-	session handlehandle	*/
ViStatus _VI_FUNC mpexxx_abort (ViSession vi)
	{
	ViStatus stat;

	/*	issues abort cmd	*/
	stat = mpexxx_Cmd(vi, ABORT);

	return stat;
	}

/*	mpexxx_set_trig_volt
		sets the trigger voltage for the supply
		vi				-	session handle
		triggerVoltage	-	value for voltage trigger setting	*/
ViStatus _VI_FUNC mpexxx_set_trig_volt (ViSession vi, ViReal64 triggerVoltage)
	{
	ViStatus stat;
	pInst inst = NULL;
        
	/*	retrieves structure ptr for lmts	*/
    if ((stat = viGetAttribute (vi, VI_ATTR_USER_DATA, &inst)) < 0)
        return stat;

	/*	chks for valid lmt struct	*/
	if (inst == NULL) 
		return MPEXXX_ERR_DRIVER_DATA;

	/*	chks range of the parameter - returns on failure	*/
	if (mpexxx_Chk_ViReal64(triggerVoltage, inst->volt_lmt, 0.0))
		return VI_ERROR_PARAMETER2;

	/*	writes the voltage set if triggered	*/
	stat = mpexxx_Cmd_ViReal64(vi, TRIG_VOLT, triggerVoltage);

	return stat;
	}

/*	mpexxx_get_trig_volt
		gets the trigger voltage for the supply
		vi				-	session handle
		triggerVoltage	-	ptr to value for voltage trigger setting	*/
ViStatus _VI_FUNC mpexxx_get_trig_volt (ViSession vi, ViReal64 *triggerVoltage)
	{
	ViStatus stat;

	/*	issues cmd and acquires response	*/
	stat = mpexxx_Get_ViReal64(vi, TRIG_VOLT, triggerVoltage);

	return stat;
	}

/*	mpexxx_set_trig_curr
		sets the trigger current for the supply
		vi				-	session handle
		triggerCurrent	-	value for current trigger setting	*/
ViStatus _VI_FUNC mpexxx_set_trig_curr (ViSession vi, ViReal64 triggerCurrent)
	{
	ViStatus stat;
	pInst inst = NULL;
        
	/*	retrieves structure ptr for lmts	*/
    if ((stat = viGetAttribute (vi, VI_ATTR_USER_DATA, &inst)) < 0)
        return stat;

	/*	chks for valid lmt struct	*/
	if (inst == NULL) 
		return MPEXXX_ERR_DRIVER_DATA;

	/*	chks range of the parameter - returns on failure	*/
	if (mpexxx_Chk_ViReal64(triggerCurrent, inst->curr_lmt, 0.0))
		return VI_ERROR_PARAMETER3;

	/*	writes the current set if triggered	*/
	stat = mpexxx_Cmd_ViReal64(vi, TRIG_CURR, triggerCurrent);

	return stat;
	}

/*	mpexxx_get_trig_curr
		gets the trigger current for the supply
		vi				-	session handle
		triggerCurrent	-	ptr to value for current trigger setting	*/
ViStatus _VI_FUNC mpexxx_get_trig_curr (ViSession vi, ViReal64 *triggerCurrent)
	{
	ViStatus stat;

	/*	issues cmd and acquires response	*/
	stat = mpexxx_Get_ViReal64(vi, TRIG_CURR, triggerCurrent);

	return stat;
	}

/*	mpexxx_trig_imm
		sends a trigger immediate
		vi	-	session handle	*/
ViStatus _VI_FUNC mpexxx_trig_imm (ViSession vi)
	{
	ViStatus stat;

	/*	issues trigger	*/
	stat = mpexxx_Cmd(vi, TRIGGER);

	return stat;
	}

/*	mpexxx_get_syst_err
		retrieves error messages from instr
		vi				-	session handle
		errorMessage	-	buf for error message	*/
ViStatus _VI_FUNC mpexxx_get_syst_err (ViSession vi, ViChar errorMessage[])
	{
	ViStatus stat;

	/*	issues cmd and acquires response	*/
	stat = mpexxx_Get_String(vi, SYST_ERR, errorMessage,  110);

	return stat;
	}

/*	mpexxx_get_syst_vers
		retrieves error version from instr
		vi				-	session handle
		versionMessage	-	buf for version message	*/
ViStatus _VI_FUNC mpexxx_get_syst_vers (ViSession vi, ViChar versionMessage[])
	{
	ViStatus stat;

	/*	issues cmd and acquires response	*/
	stat = mpexxx_Get_String(vi, SYST_VER, versionMessage, 110);

	return stat;
	}

/*	mpexxx_set_init_imm
		sends an initiate immediate
		vi	-	session handle	*/
ViStatus _VI_FUNC mpexxx_set_init_imm (ViSession vi)
	{
	ViStatus stat;

	/*	inits system for immediate trigger	*/
	stat = mpexxx_Cmd(vi, INIT_IMM);

	return stat;
	}

/*	mpexxx_set_init_cont
		sets the initiate sub-system into continous mode
		vi					-	session handle
		initiateContinuous	-	discrete for initiate continous state	*/
ViStatus _VI_FUNC mpexxx_set_init_cont (ViSession vi,
                                        ViBoolean initiateContinuous)
	{
	ViStatus stat;

	/*	chks for valid bool	*/
	if (mpexxx_Chk_ViBoolean(initiateContinuous))
		return VI_ERROR_PARAMETER2;

	/*	issues init cont cmd	*/
	stat = mpexxx_Cmd_ViBoolean(vi, INIT_CONT, initiateContinuous);

	return stat;
	}

/*	mpexxx_get_init_cont
		retrieves the state of initiate continous
		vi	-	session handle
		initiateContinuous	-	ptr to discrete for initiate continous state	*/
ViStatus _VI_FUNC mpexxx_get_init_cont (ViSession vi,
                                        ViBoolean *initiateContinuous)
	{
	ViStatus stat;

	/*	issues cmd and acquires response	*/
	stat = mpexxx_Get_Bool(vi, INIT_CONT, initiateContinuous);

	return stat;
	}

/*	mpexxx_get_stat_oper
		returns the operation register
		vi					-	session handle
		operationRegister	-	ptr to returned value	*/
ViStatus _VI_FUNC mpexxx_get_stat_oper (ViSession vi,
                                        ViInt16 *operationRegister)
	{
	ViStatus stat;

	/*	issues cmd and acquires response	*/
	stat = mpexxx_Get_ViInt16(vi, STAT_OPER, operationRegister);

	return stat;
	}

/*	mpexxx_get_stat_ques
		returns the questionable register
		vi						-	session handle
		questionableRegister	-	ptr to returned value	*/
ViStatus _VI_FUNC mpexxx_get_stat_ques (ViSession vi,
                                        ViInt16 *questionableRegister)
	{
	ViStatus stat;

	/*	issues cmd and acquires response	*/
	stat = mpexxx_Get_ViInt16(vi, STAT_QUES, questionableRegister);

	return stat;
	}

/*	mpexxx_get_meas_volt
		returns the output voltage of the supply
		vi		-	session handle
		voltage	-	ptr to the value	*/
ViStatus _VI_FUNC mpexxx_get_meas_volt (ViSession vi, ViReal64 *voltage)
	{
	ViStatus stat;

	/*	issues cmd and acquires response	*/
	stat = mpexxx_Get_ViReal64(vi, MEAS_VOLT, voltage);

	return stat;
	}

/*	mpexxx_Get_Meas_Curr
		returns the output current of the supply
		vi		-	session handle
		current	-	ptr to the value	*/
ViStatus _VI_FUNC mpexxx_get_meas_curr (ViSession vi, ViReal64 *current)
	{
	ViStatus stat;

	/*	issues cmd and acquires response	*/
	stat = mpexxx_Get_ViReal64(vi, MEAS_CURR, current);

	return stat;
	}

/*	mpexxx_set_cal_idn
		sets the identification string of the supply
		vi						-	session handle
		identificationString	-	buf for id string	*/
ViStatus _VI_FUNC mpexxx_set_cal_idn (ViSession vi,
                                      ViChar identificationString[])
	{
	ViStatus stat;

	/*	sets the id string	*/
	stat = mpexxx_Cmd_ViString(vi, CAL_IDN, identificationString);

	return stat;
	}

/*	mpexxx_get_cal_idn
		gets the identification string of the supply
		vi						-	session handle
		identificationString	-	buf for id string	*/
ViStatus _VI_FUNC mpexxx_get_cal_idn (ViSession vi,
                                      ViChar identificationString[])
	{
	ViStatus stat;

	/*	issues cmd and acquires response	*/
	stat = mpexxx_Get_String(vi, CAL_IDN, identificationString, 110);

	return stat;
	}

/*	mpexxx_cal_pwd
		transmitts password for cal
		vi			-	session handle
		password	-	password value	*/
ViStatus _VI_FUNC mpexxx_cal_pwd (ViSession vi, ViInt16 password)
	{
	ViStatus stat;

	/*	chks range of parameter	*/
	if (mpexxx_Chk_ViInt16(password, 5000, 0))
		return VI_ERROR_PARAMETER2;

	/*	sends password	*/
	stat = mpexxx_Cmd_ViInt16(vi, CAL_PWD, password);

	return stat;
	}

/*	mpexxx_set_cal_pot
		updates a pot setting - only works if cal sub-system is intiated
		vi	-	session handle
		pot		-	pot number
		val		-	pot value	*/
ViStatus _VI_FUNC mpexxx_set_cal_pot (ViSession vi, ViInt16 pot, ViInt16 value)
	{
	ViStatus stat;

	/*	chks range of parameters	*/
	if (mpexxx_Chk_ViInt16(pot, 5, 1))
		return VI_ERROR_PARAMETER2;

	if (mpexxx_Chk_ViInt16(value, 255, 0))
		return VI_ERROR_PARAMETER3;

	/*	issues pot value	*/
	stat = mpexxx_Cmd_TwoInts(vi, CAL_POTS, pot, value);

	return stat;
	}

/*	mpexxx_get_cal_pot
		returns a pot setting
		vi	-	session handle
		pot	-	pot number
		val	-	ptr to pot value	*/
ViStatus _VI_FUNC mpexxx_get_cal_pot (ViSession vi, ViInt16 pot, ViInt16 *value)
	{
	ViStatus stat;

	/*	issues cmd and acquires response	*/
	stat = mpexxx_Get_PotVal(vi, CAL_POTS, pot, value);

	return stat;
	}

/*	mpexxx_set_cal_defs
		sets defaults for supply - only works if cal sub-system is intiated
		vi	-	session handle	*/
ViStatus _VI_FUNC mpexxx_set_cal_defs (ViSession vi)
	{
	ViStatus stat;

	/*	issues cmd	*/
	stat = mpexxx_Cmd(vi, CAL_DEFS);

	return stat;
	}

/*	mpexxx_cal_end
		stops cal sub-system
		vi	-	session handle	*/
ViStatus _VI_FUNC mpexxx_cal_end (ViSession vi)
	{
	ViStatus stat;

	/*	issues cmd	*/
	stat = mpexxx_Cmd(vi, CAL_STOP);

	return stat;
	}

/*	mpexxx_get_vscl
		returns the voltage scale for the supply
		vi				-	session handle
		voltageScale	-	ptr used to return voltage scale	*/
ViStatus _VI_FUNC mpexxx_get_vscl (ViSession vi, ViReal64 *voltageScale)
	{
	ViStatus stat;

	/*	issues cmd and acquires response	*/
	stat = mpexxx_Get_ViReal64(vi, CAL_VSCL, voltageScale);

	return stat;
	}

/*	mpexxx_get_iscl
		returns the current scale for the supply
		vi				-	session handle
		currentScale	-	ptr used to return current scale	*/
ViStatus _VI_FUNC mpexxx_get_iscl (ViSession vi, ViReal64 *currentScale)
	{
	ViStatus stat;

	/*	issues cmd and acquires response	*/
	stat = mpexxx_Get_ViReal64(vi, CAL_ISCL, currentScale);

	return stat;
	}

/*	mpexxx_set_modulation_table
		sets the modulation voltage and factor for a specified row
		vi					-	session handle
		row					-	value oflocation in table
		modulationVoltage	-	value of modulation input voltage
		modulationFactor	-	value of scaling factor for modulation	*/
ViStatus _VI_FUNC mpexxx_mpexxx_set_modulation_table (ViSession vi, ViInt16 row, ViReal64 modulationVoltage, ViReal64 modulationFactor)
	{
	ViStatus stat;
	ViInt16 siz;
	ViUInt32 ret;
	char buf[50];

	/*	chks ranges of the parameters	*/
	if (mpexxx_Chk_ViInt16(row, 20, 1))
		return VI_ERROR_PARAMETER2;

	if (mpexxx_Chk_ViReal64(modulationVoltage, 2.0, 0.0))
		return VI_ERROR_PARAMETER3;

	if (mpexxx_Chk_ViReal64(modulationFactor, 1000.0, -2.0))
		return VI_ERROR_PARAMETER4;

	/*	creates cmd	*/
	siz = sprintf(buf, "%s %d(%01.3f,%01.3f)\n", pCmds[TABL_I_O], row, modulationVoltage, modulationFactor);

	/*	issues cmd	*/
	stat = viWrite(vi, buf, siz, &ret);

	return stat;
	}

/*	mpexxx_get_modulation_table
		returns the modulation voltage and factor for a specified row
		vi					-	session handle
		row					-	value of location in table
		modulationVoltage	-	ptr to modulation input voltage
		modulationFactor	-	ptr to scaling factor for modulation	*/
ViStatus _VI_FUNC mpexxx_mpexxx_get_modulation_table (ViSession vi, ViInt16 row, ViReal64 *modulationVoltage, ViReal64 *modulationFactor)
	{
	ViStatus stat;
	ViInt16 siz;
	ViUInt32	ret;
	char buf[50];
	char *ptr;

	/*	chks ranges of the parameters	*/
	if (mpexxx_Chk_ViInt16(row, 20, 1))
		return VI_ERROR_PARAMETER2;

	/*	creates cmd	*/
	siz = sprintf(buf, "%s? %d\n", pCmds[TABL_I_O], row);

	/*	issues cmd	*/
	if ((stat = viWrite(vi, buf, siz, &ret)) < VI_SUCCESS)
		return stat;

	/*	reads val from inst	*/
	memset(buf, 0x00, 50);
	if ((stat = viRead(vi, buf, sizeof(buf), &ret)) < VI_SUCCESS)
		return stat;

	/*	valid string is of the form x(y,z)	*/
	/*	this moves to y elm in string	*/
	ptr = strstr(buf, "(");
	if (ptr == VI_NULL)
		return MPEXXX_WARNING_INVALID_RESP;

	ptr++;
	/*	converts ascii to ViReal64	*/
	*modulationVoltage = atof(ptr);

	/*	this moves to z elm in string	*/
	ptr = strstr(buf, ",");
	if (ptr == VI_NULL)
		return MPEXXX_WARNING_INVALID_RESP;

	ptr++;
	/*	converts ascii to ViReal64	*/
	*modulationFactor = atof(ptr);

	/*	conds warning - we are looking for a term char	*/
	if (stat == VI_SUCCESS_TERM_CHAR)
		stat = VI_SUCCESS;

	return stat;
	}

/*	mpexxx_set_modulation_table_info
		sets the selected output modulate & modulation type
		vi				-	session handle
		controlInput	-	value of table type (none, voltage & current)
		modulationType	-	value of modulation type (multiplication & addition)	*/
ViStatus _VI_FUNC mpexxx_mpexxx_set_modulation_type (ViSession vi, ViInt16 controlInput, ViInt16 modulationType)
	{
	ViStatus stat;
	ViInt16 siz;
	ViUInt32	ret;
	char buf[50];

	/*	chks ranges of the parameters	*/
	if (mpexxx_Chk_ViInt16(controlInput, 2, 0))
		return VI_ERROR_PARAMETER2;

	if (mpexxx_Chk_ViInt16(modulationType, 1, 0))
		return VI_ERROR_PARAMETER3;

	/*	creates cmd	*/
	siz = sprintf(buf, "%s %d,%d\n", pCmds[TABL_TYP], controlInput, modulationType);

	/*	issues cmd	*/
	stat = viWrite(vi, buf, siz, &ret);

	return stat;
	}

/*	mpexxx_mpexxx_get_modulation_type
		returns the selected output modulate & modulation type
		vi				-	session handle
		controlInput	-	ptr to table type (none, voltage & current)
		modulationType	-	ptr to modulation type (multiplication & addition)	*/
ViStatus _VI_FUNC mpexxx_mpexxx_get_modulation_type (ViSession vi, ViInt16 *controlInput, ViInt16 *modulationType)
	{
	ViStatus stat;
	ViInt16 siz;
	ViUInt32	ret;
	char buf[50];
	char *ptr;

	/*	creates cmd	*/
	siz = sprintf(buf, "%s?\n", pCmds[TABL_TYP]);

	/*	issues cmd	*/
	if ((stat = viWrite(vi, buf, siz, &ret)) < VI_SUCCESS)
		return stat;

	/*	reads val from inst	*/
	memset(buf, 0x00, 50);
	if ((stat = viRead(vi, buf, sizeof(buf), &ret)) < VI_SUCCESS)
		return stat;

	/*	valid string is of the form x,y	*/
	/*	converts ascii to ViReal64	*/
	*controlInput = atoi(buf);

	/*	this moves to y elm in string	*/
	ptr = strstr(buf, ",");
	if (ptr == VI_NULL)
		return MPEXXX_WARNING_INVALID_RESP;

	ptr++;
	/*	converts ascii to ViReal64	*/
	*modulationType = atoi(ptr);

	/*	conds warning - we are looking for a term char	*/
	if (stat == VI_SUCCESS_TERM_CHAR)
		stat = VI_SUCCESS;

	return stat;
	}

/*----------------------------------------------------------------------------------
						IEEE routines for mpe instrument
  ----------------------------------------------------------------------------------*/

/*	mpexxx_Cmd_CLS
		clears the interface
		vi	-	session handle	*/
ViStatus _VI_FUNC mpexxx_set_cls (ViSession vi)
	{
	ViStatus stat;

	/*	issues cmd	*/
	stat = mpexxx_Cmd(vi, IEEE_CLS);

	return stat;
	}

/*	mpexxx_set_ese
		sets the event status enable register
		vi		-	session handle
		value	-	val to set the event status enable register	*/
ViStatus _VI_FUNC mpexxx_set_ese (ViSession vi, ViInt16 value)
	{
	ViStatus stat;

	/*	chks ranges of the parameters	*/
	if (mpexxx_Chk_ViInt16(value, 0xFF, 0))
		return VI_ERROR_PARAMETER2;

	/*	issues cmd	*/
	stat = mpexxx_Cmd_ViInt16(vi, IEEE_ESE, value);

	return stat;
	}

/*	mpexxx_get_ese
		retrieves the event status register
		vi		-	session handle
		value	-	ptr tyo value of event status reg	*/
ViStatus _VI_FUNC mpexxx_get_ese (ViSession vi, ViInt16 *value)
	{
	ViStatus stat;

	/*	issues cmd & returns val	*/
	stat = mpexxx_Get_ViInt16(vi, IEEE_ESE, value);

	return stat;
	}

/*	mpexxx_get_esr
		returns the event status register
		vi		-	session handle
		value	-	ptr to event status register value	*/
ViStatus _VI_FUNC mpexxx_get_esr (ViSession vi, ViInt16 *value)
	{
	ViStatus stat;

	/*	issues cmd & returns val	*/
	stat = mpexxx_Get_ViInt16(vi, IEEE_ESR, value);

	return stat;
	}

/*	mpexxx_get_idn
		returns the identification string
		vi						-	session handle
		identificationString	-	id buffer	*/
ViStatus _VI_FUNC mpexxx_get_idn (ViSession vi, ViChar identificationString[])
	{
	ViStatus stat;

	/*	issues cmd & returns val	*/
	stat = mpexxx_Get_String(vi, IEEE_IDN, identificationString, 110);

	return stat;
	}

/*	mpexxx_set_rcl
		recalls the specified mem state
		vi		-	session handle
		value	-	mem address value	*/
ViStatus _VI_FUNC mpexxx_set_rcl (ViSession vi, ViInt16 value)
	{
	ViStatus stat;

	/*	chks ranges of the parameters	*/
	if (mpexxx_Chk_ViInt16(value, 99, 0))
		return VI_ERROR_PARAMETER2;

	/*	issues cmd	*/
	stat = mpexxx_Cmd_ViInt16(vi, IEEE_RCL, value);

	return stat;
	}

/*	mpexxx_Cmd_RST
		resets system according to manual
		vi	-	session handle	*/
ViStatus _VI_FUNC mpexxx_set_rst (ViSession vi)
	{
	ViStatus stat;

	/*	issues cmd	*/
	stat = mpexxx_Cmd(vi, IEEE_RST);

	return stat;
	}

/*	mpexxx_Cmd_SAV
		saves to a specified state
		vi		-	session handle
		value	-	memory location to save	*/
ViStatus _VI_FUNC mpexxx_set_sav (ViSession vi, ViInt16 value)
	{
	ViStatus stat;

	/*	chks ranges of the parameters	*/
	if (mpexxx_Chk_ViInt16(value, 99, 0))
		return VI_ERROR_PARAMETER2;

	/*	issues cmd	*/
	stat = mpexxx_Cmd_ViInt16(vi, IEEE_SAV, value);

	return stat;
	}

/*	mpexxx_Cmd_SRE
		sets the service request register
		vi		-	session handle
		value	-	value to set the service request register	*/
ViStatus _VI_FUNC mpexxx_set_sre (ViSession vi, ViInt16 value)
	{
	ViStatus stat;

	/*	chks ranges of the parameters	*/
	if (mpexxx_Chk_ViInt16(value, 0xFF, 0))
		return VI_ERROR_PARAMETER2;

	/*	issues cmd	*/
	stat = mpexxx_Cmd_ViInt16(vi, IEEE_SRE, value);

	return stat;
	}

/*	mpexxx_get_sre
		returns the service request enable register
		vi		-	session handle
		value	-	ptr to service request enable register value	*/
ViStatus _VI_FUNC mpexxx_get_sre (ViSession vi, ViInt16 *value)
	{
	ViStatus stat;

	/*	issues cmd & returns val	*/
	stat = mpexxx_Get_ViInt16(vi, IEEE_SRE, value);

	return stat;
	}

/*	mpexxx_get_stb
		returns the service byte register
		vi		-	session handle
		value	-	ptr to status byte register value	*/
ViStatus _VI_FUNC mpexxx_get_stb (ViSession vi, ViInt16 *value)
	{
	ViStatus stat;

	/*	issues cmd & returns val	*/
	stat = mpexxx_Get_ViInt16(vi, IEEE_STB, value);

	return stat;
	}

/*	mpexxx_reset
		resets instrument to power on state
		vi		-	session handle	*/
ViStatus _VI_FUNC mpexxx_reset (ViSession vi)
	{
	ViStatus stat;

	/*	issues cmd & returns val	*/
	stat = mpexxx_Cmd(vi, IEEE_RST);

	return stat;
	}

/*	mpexxx_self_test
		performs self test - no self test (conform pnp spec
		vi				-	session handle
		errorCode		-	ptr to error code
		errorMessage	-	buf for error message	*/
ViStatus _VI_FUNC mpexxx_self_test (ViSession vi, ViInt32 *Error_Code,
                                   ViChar Error_Message[])
	{
	return VI_WARN_NSUP_SELF_TEST;    
	}

/*	mpexxx_error_query
		retrieve error buffer
		vi				-	session handle
		errorCode		-	ptr to error code
		errorMessage	-	buf for error message	*/
ViStatus _VI_FUNC mpexxx_error_query (ViSession vi, ViInt32 *Error_Code,
                    ViChar Error_Message[])
	{
    ViStatus stat;

	/*	issues cmd and acquires response	*/
	stat = mpexxx_Get_String(vi, SYST_ERR, Error_Message,  110);

	/*	issues cmd and acquires response	*/
	*Error_Code = atol(Error_Message);

    return stat;
	}

/*	mpexxx_revision_query
		returns the queries
		vi							-	session handle
		instrumentDriverRevision	-	buf for driver rev
		firmwareRevision			-	buf for firmware rev	*/
ViStatus _VI_FUNC mpexxx_revision_query (ViSession vi, ViChar instrumentDriverRevision[], ViChar firmwareRevision[])
	{
	ViStatus stat;
	ViChar buf[110];

	/*	retrieves version string	*/
	if ((stat = mpexxx_get_syst_vers(vi, buf))  < VI_SUCCESS)
		return stat;

	strcpy(firmwareRevision, buf);
	sprintf(instrumentDriverRevision, "%s", mpexxx_REV);

	return stat;
	}

/*	mpexxx_error_message
		returns the service byte register
		vi			-	session handle
		statusCode	-	status code
		message		-	buf for error message	*/
ViStatus _VI_FUNC mpexxx_error_message (ViSession vi, ViStatus statusCode, ViChar message[])
	{
    ViStatus stat;
    ViInt16 cnt;
    static mpexxx_tStringValPair statusDescArray[] = {
        {VI_WARN_NSUP_ID_QUERY,		"WARNING: ID Query not supported"},
        {VI_WARN_NSUP_RESET,		"WARNING: Reset not supported"},
        {VI_WARN_NSUP_SELF_TEST,	"WARNING: Self-test not supported"},
        {VI_WARN_NSUP_ERROR_QUERY,	"WARNING: Error Query not supported"},     
        {VI_WARN_NSUP_REV_QUERY,	"WARNING: Revision Query not supported"},
        {MPEXXX_WARNING_MODEL_DATA,	"WARNING: Command incompatible with instrument"},
        {VI_ERROR_PARAMETER1,		"ERROR: Parameter 1 out of range"},
        {VI_ERROR_PARAMETER2,		"ERROR: Parameter 2 out of range"},
        {VI_ERROR_PARAMETER3,		"ERROR: Parameter 3 out of range"},
        {VI_ERROR_PARAMETER4,		"ERROR: Parameter 4 out of range"},
        {VI_ERROR_PARAMETER5,		"ERROR: Parameter 5 out of range"},
        {VI_ERROR_PARAMETER6,		"ERROR: Parameter 6 out of range"},
        {VI_ERROR_PARAMETER7,		"ERROR: Parameter 7 out of range"},
        {VI_ERROR_PARAMETER8,		"ERROR: Parameter 8 out of range"},
        {VI_ERROR_FAIL_ID_QUERY,	"ERROR: Identification query failed"},
        {VI_ERROR_INV_RESPONSE,		"ERROR: Interpreting instrument response"},
        {MPEXXX_ERR_DRIVER_DATA,	"ERROR: Incomplete or incorrect initialization!"},
        {VI_NULL, VI_NULL}
		};

    stat = viStatusDesc(vi, statusCode, message);
    if (stat == VI_WARN_UNKNOWN_STATUS)
		{
        for (cnt = 0; statusDescArray[cnt].stringName; cnt++)
			{
            if (statusDescArray[cnt].stringVal == statusCode)
				{
                sprintf(message, "%s", statusDescArray[cnt].stringName);
                return VI_SUCCESS;
				}
			}
        sprintf(message, "Unknown Error %X", statusCode);
        return VI_WARN_UNKNOWN_STATUS;
		}
    
    return VI_SUCCESS;
	}

/*----------------------------------------------------------------------------------
					clean-up routines for mpe instrument
  ----------------------------------------------------------------------------------*/

/*	mpexxx_initCleanUp
		closes session and frees memory when failure occurs
		instRMSess	-	resource manager session
		vi			-	specific session
		curStatus	-	status of failure	*/
ViStatus _VI_FUNC mpexxx_initCleanUp(ViSession instRMSess, ViPSession vi, ViStatus curStatus)
	{
	pInst inst = NULL;
        
	/*	retrieves structure ptr for lmts	*/
    if (viGetAttribute (*vi, VI_ATTR_USER_DATA, &inst) >= 0)
		{
		/*	frees memory	*/
		if (inst != NULL) 
			free(inst);
		}

	/*	closes & defaults vars	*/
	inst = NULL;
	viClose(*vi);
	viClose(instRMSess);
	*vi = VI_NULL;

	return curStatus;
	}

/*----------------------------------------------------------------------------------
					low output routines for mpe instrument
  ----------------------------------------------------------------------------------*/

/*	mpexxx_Cmd_ViReal64
		creates and issues a cmd for a floating point number
		vi	-	session handle
		cmd	-	index into cmd struct
		val	-	val to set for cmd	*/
ViStatus _VI_FUNC mpexxx_Cmd_ViReal64(ViSession vi, ViInt16 cmd, ViReal64 val)
	{
	ViStatus stat;
	ViInt16 siz;
	ViUInt32 ret;
	char buf[50];

	/*	creates cmd	*/
	siz = sprintf(buf, "%s %1.3lf\n", pCmds[cmd], val);

	/*	issues cmd	*/
	stat = viWrite(vi, buf, siz, &ret);

	return stat;
	}

/*	mpexxx_Cmd_ViInt16
		creates and issues a cmd for an integer
		vi	-	session handle
		cmd	-	index into cmd struct
		val	-	val to set for cmd	*/
ViStatus _VI_FUNC mpexxx_Cmd_ViInt16(ViSession vi, ViInt16 cmd, ViInt16 val)
	{
	ViStatus	stat;
	ViInt16		siz;
	ViUInt32 ret;
	char		buf[50];

	/*	creates cmd	*/
	siz = sprintf(buf, "%s %d\n", pCmds[cmd], val);

	/*	issues cmd	*/
	stat = viWrite(vi, buf, siz, &ret);

	return stat;
	}

/*	mpexxx_Cmd_TwoInts
		creates and issues a cmd for two integers (pot cmd)
		vi		-	session handle
		cmd		-	index into cmd struct
		val1	-	val1 to set for cmd
		val2	-	val2 to set for cmd	*/
ViStatus _VI_FUNC mpexxx_Cmd_TwoInts(ViSession vi, ViInt16 cmd, ViInt16 val1, ViInt16 val2)
	{
	ViStatus	stat;
	ViInt16		siz;
	ViUInt32 ret;
	char		buf[50];

	/*	creates cmd	*/
	siz = sprintf(buf, "%s %d,%d\n", pCmds[cmd], val1, val2);

	/*	issues cmd	*/
	stat = viWrite(vi, buf, siz, &ret);

	return stat;
	}

/*	mpexxx_Cmd_ViBoolean
		creates and issues a cmd for a boolean
		vi	-	session handle
		cmd	-	index into cmd struct
		val	-	val1 to set for cmd	*/
ViStatus _VI_FUNC mpexxx_Cmd_ViBoolean(ViSession vi, ViInt16 cmd, ViBoolean val)
	{
	ViStatus stat;
	ViInt16 siz;
	ViUInt32 ret;
	char buf[50];

	/*	creates cmd	*/
	siz = sprintf(buf, "%s %s\n", pCmds[cmd], (val == VI_TRUE) ? "ON" : "OFF");

	/*	issues cmd	*/
	stat = viWrite(vi, buf, siz, &ret);

	return stat;
	}

/*	mpexxx_Cmd_ViString
		creates and issues a cmd for a string (cal idn)
		vi		-	session handle
		cmd		-	index into cmd struct
		pBuf	-	val1 to set for cmd	*/
ViStatus _VI_FUNC mpexxx_Cmd_ViString(ViSession vi, ViInt16 cmd, ViString pBuf)
	{
	ViStatus stat;
	ViInt16 siz;
	ViUInt32	ret;
	char buf[100];

	/*	creates cmd	*/
	siz = sprintf(buf, "%s %s\n", pCmds[cmd], pBuf);

	/*	issues cmd	*/
	stat = viWrite(vi, buf, siz, &ret);

	return stat;
	}

/*	mpexxx_Cmd
		creates and issues a cmd
		vi		-	session handle
		cmd		-	index into cmd struct
		pBuf	-	val1 to set for cmd	*/
ViStatus _VI_FUNC mpexxx_Cmd(ViSession vi, ViInt16 cmd)
	{
	ViStatus stat;
	ViInt16 siz;
	ViUInt32	ret;
	char buf[50];

	/*	creates cmd	*/
	siz = sprintf(buf, "%s\n", pCmds[cmd]);

	/*	issues cmd	*/
	stat = viWrite(vi, buf, siz, &ret);

	return stat;
	}

/*----------------------------------------------------------------------------------
						query routines for mpe instrument
  ----------------------------------------------------------------------------------*/

/*	mpexxx_Get_MAX
		creates and issues a cmd for max value of cmd
		vi	-	session handle
		cmd	-	index into cmd struct
		val	-	ptr to return val	*/
ViStatus _VI_FUNC mpexxx_Get_MAX(ViSession vi, ViInt16 cmd, ViPReal64 val)
	{
	ViStatus stat;
	ViInt16 siz;
	ViUInt32	ret;
	char buf[50];

	/*	creates cmd	*/
	siz = sprintf(buf, "%s? MAX\n", pCmds[cmd]);

	/*	issues cmd	*/
	if ((stat = viWrite(vi, buf, siz, &ret)) < VI_SUCCESS)
		return stat;

	/*	reads val from inst	*/
	memset(buf, 0x00, 50);
	if ((stat = viRead(vi, buf, sizeof(buf), &ret)) < VI_SUCCESS)
		return stat;

	/*	converts ascii to ViReal64	*/
	*val = atof(buf);

	/*	conds warning - we are looking for a term char	*/
	if (stat == VI_SUCCESS_TERM_CHAR)
		stat = VI_SUCCESS;

	return stat;
	}

/*	mpexxx_Get_MIN
		creates and issues a cmd for min value of cmd
		vi	-	session handle
		cmd	-	index into cmd struct
		val	-	ptr to return val	*/
ViStatus _VI_FUNC mpexxx_Get_MIN(ViSession vi, ViInt16 cmd, ViPReal64 val)
	{
	ViStatus stat;
	ViInt16 siz;
	ViUInt32	ret;
	char buf[50];

	/*	creates cmd	*/
	siz = sprintf(buf, "%s? MIN\n", pCmds[cmd]);

	/*	issues cmd	*/
	if ((stat = viWrite(vi, buf, siz, &ret)) < VI_SUCCESS)
		return stat;

	/*	reads val from inst	*/
	memset(buf, 0x00, 50);
	if ((stat = viRead(vi, buf, sizeof(buf), &ret)) < VI_SUCCESS)
		return stat;

	/*	converts ascii to ViReal64	*/
	*val = atof(buf);

	/*	conds warning - we are looking for a term char	*/
	if (stat == VI_SUCCESS_TERM_CHAR)
		stat = VI_SUCCESS;

	return stat;
	}

/*	mpexxx_Get_ViReal64
		creates and issues a cmd for to return a real val
		vi	-	session handle
		cmd	-	index into cmd struct
		val	-	ptr to return val	*/
ViStatus _VI_FUNC mpexxx_Get_ViReal64(ViSession vi, ViInt16 cmd, ViPReal64 val)
	{
	ViStatus stat;
	ViInt16 siz;
	ViUInt32	ret;
	char buf[50];

	/*	creates cmd	*/
	siz = sprintf(buf, "%s?\n", pCmds[cmd]);

	/*	issues cmd	*/
	if ((stat = viWrite(vi, buf, siz, &ret)) < VI_SUCCESS)
		return stat;

	/*	reads val from inst	*/
	memset(buf, 0x00, 50);
	if ((stat = viRead(vi, buf, sizeof(buf), &ret)) < VI_SUCCESS)
		return stat;

	/*	converts ascii to ViReal64	*/
	*val = atof(buf);

	/*	conds warning - we are looking for a term char	*/
	if (stat == VI_SUCCESS_TERM_CHAR)
		stat = VI_SUCCESS;

	return stat;
	}

/*	mpexxx_Get_ViInt16
		creates and issues a cmd for to return a int val
		vi	-	session handle
		cmd	-	index into cmd struct
		val	-	ptr to return val	*/
ViStatus _VI_FUNC mpexxx_Get_ViInt16(ViSession vi, ViInt16 cmd, ViPInt16 val)
	{
	ViStatus stat;
	ViInt16 siz;
	ViUInt32	ret;
	char buf[50];

	/*	creates cmd	*/
	siz = sprintf(buf, "%s?\n", pCmds[cmd]);

	/*	issues cmd	*/
	if ((stat = viWrite(vi, buf, siz, &ret)) < VI_SUCCESS)
		return stat;

	/*	reads val from inst	*/
	memset(buf, 0x00, 50);
	if ((stat = viRead(vi, buf, sizeof(buf), &ret)) < VI_SUCCESS)
		return stat;

	/*	converts ascii to ViInt16	*/
	*val = atoi(buf);

	/*	conds warning - we are looking for a term char	*/
	if (stat == VI_SUCCESS_TERM_CHAR)
		stat = VI_SUCCESS;

	return stat;
	}

/*	mpexxx_Get_PotVal
		creates and issues a cmd for to return a int val
		vi	-	session handle
		cmd	-	index into cmd struct
		pot	-	pot number
		val	-	ptr to return val	*/
ViStatus _VI_FUNC mpexxx_Get_PotVal(ViSession vi, ViInt16 cmd, ViInt16 pot, ViPInt16 val)
	{
	ViStatus stat;
	ViInt16 siz;
	ViUInt32	ret;
	char buf[50];

	/*	creates cmd	*/
	siz = sprintf(buf, "%s? %d\n", pCmds[cmd], pot);

	/*	issues cmd	*/
	if ((stat = viWrite(vi, buf, siz, &ret)) < VI_SUCCESS)
		return stat;

	/*	reads val from inst	*/
	memset(buf, 0x00, 50);
	if ((stat = viRead(vi, buf, sizeof(buf), &ret)) < VI_SUCCESS)
		return stat;

	/*	converts ascii to ViInt16	*/
	*val = atoi(buf);

	/*	conds warning - we are looking for a term char	*/
	if (stat == VI_SUCCESS_TERM_CHAR)
		stat = VI_SUCCESS;

	return stat;
	}

/*	mpexxx_Get_Bool
		creates and issues a cmd for to return a bool val
		vi	-	session handle
		cmd	-	index into cmd struct
		val	-	ptr to return val	*/
ViStatus _VI_FUNC mpexxx_Get_Bool(ViSession vi, ViInt16 cmd, ViPBoolean val)
	{
	ViStatus stat;
	ViInt16 siz;
	ViUInt32	ret;
	char buf[50];

	/*	creates cmd	*/
	siz = sprintf(buf, "%s?\n", pCmds[cmd]);

	/*	issues cmd	*/
	if ((stat = viWrite(vi, buf, siz, &ret)) < VI_SUCCESS)
		return stat;

	/*	reads val from inst	*/
	memset(buf, 0x00, 50);
	if ((stat = viRead(vi, buf, sizeof(buf), &ret)) < VI_SUCCESS)
		return stat;

	/*	converts ascii to ViPBoolean	*/
	siz = atoi(buf);
	if (siz == 1)
		*val = VI_TRUE;
	else
		*val = VI_FALSE;

	/*	conds warning - we are looking for a term char	*/
	if (stat == VI_SUCCESS_TERM_CHAR)
		stat = VI_SUCCESS;

	return stat;
	}

/*	mpexxx_Get_String
		creates and issues a cmd for to return a bool val
		vi		-	session handle
		cmd		-	index into cmd struct
		pBuf	-	ptr to buffer
		buf_siz	-	size of buffer	*/
ViStatus _VI_FUNC mpexxx_Get_String(ViSession vi, ViInt16 cmd, ViString pBuf, ViInt16 buf_siz)
	{
	ViStatus stat;
	ViInt16 siz;
	ViUInt32 ret;
	char buf[110];

	/*	creates cmd	*/
	siz = sprintf(buf, "%s?\n", pCmds[cmd]);

	/*	issues cmd	*/
	if ((stat = viWrite(vi, buf, siz, &ret)) < VI_SUCCESS)
		return stat;

	/*	reads val from inst	*/
	memset(buf, 0x00, 110);
	if ((stat = viRead(vi, buf, sizeof(buf), &ret)) < VI_SUCCESS)
		return stat;

	/*	copies into buffer not exceeding 110	*/
	if (buf_siz > 110)
		strncpy(pBuf, buf, 110);
	else
		strncpy(pBuf, buf, ret + 1);
	
	/*	conds warning - we are looking for a term char	*/
	if (stat == VI_SUCCESS_TERM_CHAR)
		stat = VI_SUCCESS;

	return stat;
	}

/*----------------------------------------------------------------------------------
						val approval routines for mpe instrument
  ----------------------------------------------------------------------------------*/

/*	mpexxx_Chk_ViBoolean
		verifies val is a bool
		val	-	val for approval	*/
ViBoolean _VI_FUNC mpexxx_Chk_ViBoolean(ViBoolean val)
	{
	if (val == VI_TRUE || val == VI_FALSE)
		return VI_FALSE;
	return VI_TRUE;
	}

/*	mpexxx_Chk_ViInt16
		verifies val is in specified range
		val	-	val for approval
		max	-	max val
		min	-	min val	*/
ViBoolean _VI_FUNC mpexxx_Chk_ViInt16(ViInt16 val, ViInt16 max, ViInt16 min)
	{
	if (val > max || val < min)
		return VI_TRUE;
	return VI_FALSE;
	}

/*	mpexxx_Chk_ViInt32
		verifies val is in specified range
		val	-	val for approval
		max	-	max val
		min	-	min val	*/
ViBoolean _VI_FUNC mpexxx_Chk_ViInt32(ViInt32 val, ViInt32 max, ViInt32 min)
	{
	if (val > max || val < min)
		return VI_TRUE;
	return VI_FALSE;
	}

/*	mpexxx_Chk_ViReal64
		verifies val is in specified range
		val	-	val for approval
		max	-	max val
		min	-	min val	*/
ViBoolean _VI_FUNC mpexxx_Chk_ViReal64(ViReal64 val, ViReal64 max, ViReal64 min)
	{
	if (val > max || val < min)
		return VI_TRUE;
	return VI_FALSE;
	}